/**
 * Copyright (c) 2020, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { TestableComponentInterface } from "@wso2is/core/models";
import { EncodeDecodeUtils } from "@wso2is/core/utils";
import { CodeEditor } from "@wso2is/react-components";
import React, { FunctionComponent, ReactElement, ReactNode, useEffect, useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import { Button, Card, Divider, DropdownItemProps, Form, Icon } from "semantic-ui-react";
import {
    angularSDKIntegrationCode,
    javascriptSDKIntegrationCode,
    reactSDKIntegrationCode
} from "./code-blocks";
import { SDKMeta } from "./meta";
import { SupportedTechnologyTypes } from "./models";
import {
    ApplicationInterface,
    ApplicationTemplateInterface
} from "../../../../features/applications";
import { AppState, store, toggleHelpPanelVisibility } from "../../../../features/core";
import { Config } from "../../../../features/core/configs";
import { VerticalStepper } from "../../../components/component-extensions/application/vertical-stepper";

interface IntegrateSDKsPropsInterface extends TestableComponentInterface {
    application: ApplicationInterface;
    technology: SupportedTechnologyTypes;
    template: ApplicationTemplateInterface;
    inboundProtocolConfig: any;
}

/**
 * Integrate SDKs to Single Page Applications.
 *
 * @param {IntegrateSDKsPropsInterface} props - Props injected into the component.
 * @return {React.ReactElement}
 */
export const IntegrateSDKs: FunctionComponent<IntegrateSDKsPropsInterface> = (
    props: IntegrateSDKsPropsInterface
): ReactElement => {

    const {
        inboundProtocolConfig,
        technology,
        [ "data-testid" ]: testId
    } = props;

    const dispatch = useDispatch();

    const helpPanelVisibilityGlobalState: boolean = useSelector((state: AppState) => state.helpPanel.visibility);

    const deploymentConfigs = store.getState().config.deployment;
    const [ SDKInitConfig, setSDKInitConfig ] = useState(undefined);
    const [ configuredCallbacks, setConfiguredCallbacks ] = useState<DropdownItemProps[]>([]);
    const [ selectedLoginCallBack, setSelectedLoginCallBack ] = useState<any>("");
    const [ selectedLogoutCallBack, setSelectedLogoutCallBack ] = useState<any>("");
    const [ isSidePanelOpen, setIsSidePanelOpen ] = useState<boolean>(false);

    useEffect(() => {

        if (!inboundProtocolConfig?.oidc?.callbackURLs
            || !Array.isArray(inboundProtocolConfig.oidc.callbackURLs)
            || inboundProtocolConfig.oidc.callbackURLs.length < 1) {

            return;
        }

        const callbacks: string[] = EncodeDecodeUtils.decodeURLRegex(inboundProtocolConfig.oidc.callbackURLs[ 0 ]);

        if (callbacks && Array.isArray(callbacks) && callbacks.length > 1) {
            callbacks.forEach((url: string) => {
                setConfiguredCallbacks((prevItems) => [
                    ...prevItems,
                    {
                        key: url,
                        text: url,
                        value: url
                    }
                ]);
            });
        }
    }, [ inboundProtocolConfig ]);

    useEffect(() => {
        if (!inboundProtocolConfig?.oidc) {
            return;
        }

        const configs = {
            clientID: inboundProtocolConfig.oidc.clientId,
            signInRedirectURL: (configuredCallbacks.length > 1)
                ? selectedLoginCallBack
                : inboundProtocolConfig.oidc.callbackURLs[ 0 ],
            signOutRedirectURL: (configuredCallbacks.length > 1)
                ? selectedLogoutCallBack
                : inboundProtocolConfig.oidc.callbackURLs[ 0 ],
            serverOrigin: Config.getDeploymentConfig().serverHost
        };

        setSDKInitConfig(configs);
    }, [ inboundProtocolConfig, configuredCallbacks, selectedLoginCallBack, selectedLogoutCallBack ]);

    /**
     * Generates integration code blocks.
     *
     * @param {SupportedTechnologyTypes} technology - Integration technology.
     * @return {React.ReactNode}
     */
    const generateIntegrationCode = (technology: SupportedTechnologyTypes): ReactNode => {

        if (!SDKInitConfig) {
            return null;
        }

        if (technology === SupportedTechnologyTypes.REACT) {
            return reactSDKIntegrationCode({
                clientID: SDKInitConfig?.clientID,
                serverOrigin: SDKInitConfig?.serverOrigin,
                signInRedirectURL:  (configuredCallbacks.length > 1)
                    ? selectedLoginCallBack
                    : SDKInitConfig?.signInRedirectURL,
                signOutRedirectURL: (configuredCallbacks.length > 1)
                    ? selectedLogoutCallBack
                    : SDKInitConfig?.signOutRedirectURL
            });
        }

        if (technology === SupportedTechnologyTypes.JAVASCRIPT) {
            return javascriptSDKIntegrationCode({
                clientID: SDKInitConfig?.clientID,
                serverOrigin: SDKInitConfig?.serverOrigin,
                signInRedirectURL:  (configuredCallbacks.length > 1)
                    ? selectedLoginCallBack
                    : SDKInitConfig?.signInRedirectURL,
                signOutRedirectURL: (configuredCallbacks.length > 1)
                    ? selectedLogoutCallBack
                    : SDKInitConfig?.signOutRedirectURL
            });
        }

        if (technology === SupportedTechnologyTypes.ANGULAR) {
            return angularSDKIntegrationCode({
                clientID: SDKInitConfig?.clientID,
                serverOrigin: SDKInitConfig?.serverOrigin,
                signInRedirectURL:  (configuredCallbacks.length > 1)
                    ? selectedLoginCallBack
                    : SDKInitConfig?.signInRedirectURL,
                signOutRedirectURL: (configuredCallbacks.length > 1)
                    ? selectedLogoutCallBack
                    : SDKInitConfig?.signOutRedirectURL
            });
        }

        return null;
    };

    const generateConfigureStep = (technology: SupportedTechnologyTypes): ReactNode => {

        if (technology === SupportedTechnologyTypes.REACT) {
            return (
                <>
                    <p>
                        Run below command to install the React SDK from the npm registry. Click
                        <a href={ SDKMeta.javascript.readme } target="_blank" rel="noopener noreferrer"> here </a>
                        to learn more.
                    </p>
                    <code className="code-block command dark">
                        npm install --save @asgardeo/oidc-js
                    </code>
                </>
            );
        }

        if (technology === SupportedTechnologyTypes.JAVASCRIPT) {
            return (
                <>
                    <p>Add the following script in to the index.html of your application.</p>
                    <code className="code-block command dark">
                        { `<script src="${ SDKMeta.javascript.cdn }"></script>` }
                    </code>
                </>
            );
        }

        if (technology === SupportedTechnologyTypes.ANGULAR) {
            return (
                <>
                    <p>
                        Run below command to install the Angular SDK from npm registry. Click
                        <a href={ SDKMeta.javascript.readme } target="_blank" rel="noopener noreferrer"> here </a>
                        to learn more.
                    </p>
                    <code className="code-block command dark">
                        npm install --save @asgardeo/angular
                    </code>
                </>
            );
        }

        return null;
    };

    const steps = [
            {
                stepContent: generateConfigureStep(technology),
                stepTitle: "Configure SDK"
            },
            {
                stepContent: (
                    <>
                        <p>Copy and paste below code at the relevant places in your application.</p>
                        {
                            (configuredCallbacks.length > 1) && (
                                <>
                                    <Form>
                                        <Form.Group widths="3">
                                            <Form.Select
                                                fluid
                                                onChange={ (value, data) => {
                                                    setSelectedLoginCallBack(data.value);
                                                } }
                                                label="Select login callback URL:"
                                                options={ configuredCallbacks }
                                                placeholder="Login Callback URL"
                                            />
                                        </Form.Group>
                                        <Form.Group widths="3">
                                            <Form.Select
                                                fluid
                                                onChange={ (value, data) => {
                                                    setSelectedLogoutCallBack(data.value);
                                                } }
                                                label="Select logout callback URL:"
                                                options={ configuredCallbacks }
                                                placeholder="Logout Callback URL"
                                            />
                                        </Form.Group>
                                    </Form>
                                    <Divider hidden/>
                                </>
                            )
                        }
                        <div className="code-segment">
                            <>
                                <div
                                    data-testid={ `${ testId }-angular-config` }
                                    className="action-buttons"
                                >
                                    <Button basic size="tiny" className="copy-config" icon>
                                        <Icon name="copy"/>
                                    </Button>
                                    <Button className="download-config" size="tiny" basic onClick={ () => {
                                        dispatch(toggleHelpPanelVisibility(true));
                                        setIsSidePanelOpen(true);
                                        // TODO: IMPORTANT Remove this once scroll is fixed.
                                        window.scrollTo(0, 0);
                                    } }>
                                        Configuration Help
                                    </Button>
                                </div>
                                <CodeEditor
                                    showLineNumbers
                                    language={
                                        (technology === SupportedTechnologyTypes.JAVASCRIPT)
                                            ? "javascript"
                                            : "typescript"
                                    }
                                    sourceCode={ generateIntegrationCode(technology) }
                                    options={ {
                                        lineWrapping: true
                                    } }
                                    theme="dark"
                                    readOnly={ true }
                                />
                            </>
                        </div>
                        <Divider/>
                        <Button
                            basic
                            floated="left"
                            className="api-ref"
                            size="tiny"
                            color="blue"
                            onClick={ () => window.open(SDKMeta.javascript.apis, "_blank")
                            }
                        >
                            SDK API Reference <Icon className="ml-1" name="external"/>
                        </Button>
                    </>
                ),
                stepTitle:
                    "Add Asgardeo to your app"
            }
        ]
    ;

    return (
        <VerticalStepper
            alwaysOpen
            isSidePanelOpen
            stepContent={ steps }
            isNextEnabled={ technology !== undefined }
        />
    );
};
