/**
 * Copyright (c) 2020, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import { TestableComponentInterface } from "@wso2is/core/models";
import { Heading, LinkButton, Text } from "@wso2is/react-components";
import React, { FunctionComponent, ReactElement, useState } from "react";
import { Divider, Grid, Header, Icon } from "semantic-ui-react";
import { IntegrateSDKs } from "./integrate-sdks";
import { TryoutSamples } from "./tryout-samples";
import { ApplicationInterface, ApplicationTemplateInterface } from "../../../../features/applications/models";
import { HelpPanelOverview } from "../../../components/component-extensions/application/overview";
import { QuickStartModes } from "../../shared";
import { SupportedTechnologyTypes } from "./models";
import { TechnologySelection } from "./technology-selection";
import { ApplicationManagementConstants } from "../../../../features/applications/constants";
import isEmpty from "lodash/isEmpty";

/**
 * Prop types of the component.
 */
interface SinglePageApplicationQuickStartPropsInterface extends TestableComponentInterface {
    application: ApplicationInterface;
    inboundProtocolConfig: any;
    template: ApplicationTemplateInterface;
}

/**
 * Quick start content for the Single page application template.
 *
 * @param {SinglePageApplicationQuickStartPropsInterface} props - Props injected into the component.
 * @return {React.ReactElement}
 */
const SinglePageApplicationQuickStart: FunctionComponent<SinglePageApplicationQuickStartPropsInterface> = (
    props: SinglePageApplicationQuickStartPropsInterface
): ReactElement => {

    const {
        application,
        inboundProtocolConfig,
        template,
        [ "data-testid" ]: testId
    } = props;

    const urlSearchParams: URLSearchParams = new URLSearchParams(location.search);
    const isNewApplication: boolean = urlSearchParams.get(
        ApplicationManagementConstants.APP_STATE_URL_SEARCH_PARAM_KEY
    ) === ApplicationManagementConstants.APP_STATE_URL_SEARCH_PARAM_VALUE;

    const [ selectedIntegration, setSelectedIntegration ] = useState<QuickStartModes>(QuickStartModes.INTEGRATE);
    const [ selectedTechnology, setSelectedTechnology ] = useState<SupportedTechnologyTypes>(undefined);

    const handleIntegrateSelection = (selection: QuickStartModes): void => {
        setSelectedIntegration(selection);
    };

    const resolveQuickStartMode = (): ReactElement => {

        switch (selectedIntegration) {
            case QuickStartModes.INTEGRATE:
                return (
                    <IntegrateSDKs
                        application={ application }
                        template={ template }
                        technology={ selectedTechnology }
                        inboundProtocolConfig={ inboundProtocolConfig }
                    />
                );
            case QuickStartModes.SAMPLES:
                return (
                    <TryoutSamples
                        application={ application }
                        template={ template }
                        technology={ selectedTechnology }
                        inboundProtocolConfig={ inboundProtocolConfig }
                    />
                );
            default:
                return null;
        }
    };

    /**
     * Resolves application name to be shown at initial creation.
     * Suffixes the `application` part if required.
     *
     * @param {string} name - App name.
     * @return {React.ReactElement}
     */
    const resolveApplicationName = (name: string): ReactElement => {

        const nameTokens: string[] = name.split(" ");
        const matchers: string[] = [ "app", "application" ];
        let requireNameSuffix: boolean = true;

        if (Array.isArray(nameTokens) && !isEmpty(nameTokens)) {
            const ending: string = nameTokens[ nameTokens.length - 1 ];

            for (const matcher of matchers) {
                if (ending.toLocaleLowerCase() === matcher) {
                    requireNameSuffix = false;
                    break;
                }
            }
        }

        return (
            <>
                <strong>{ application.name }</strong> { requireNameSuffix && <> application</> } has been registered!
            </>
        );
    };

    const resetTabState = () => {
        setSelectedTechnology(undefined);
        setSelectedIntegration(QuickStartModes.INTEGRATE);
    };

    return (
        <Grid data-testid={ testId } className="ml-0 mr-0">
            {
                !selectedTechnology
                    ? (
                        <Grid.Row className="technology-selection-wrapper" textAlign="center">
                            {
                                isNewApplication && (
                                    <Grid.Column width={ 16 }>
                                        <Heading as="h4" className="new-app-greeting mt-3 mb-4">
                                            { resolveApplicationName(application.name) }
                                        </Heading>
                                    </Grid.Column>
                                )
                            }

                            <Grid.Column width={ 16 }>
                                <TechnologySelection
                                    onSelectedTechnologyChange={
                                        (technology: SupportedTechnologyTypes) => setSelectedTechnology(technology)
                                    }
                                />
                            </Grid.Column>.
                        </Grid.Row>
                    )
                    : (
                        <>
                            <Grid.Row>
                                <Grid.Column width={ 13 }>
                                    <HelpPanelOverview
                                        technology={ selectedTechnology }
                                        applicationType={ template.id }
                                        application={ application }
                                        inboundProtocols={ application?.inboundProtocols }
                                        onBackButtonClick={ () => resetTabState() }
                                        handleIntegrateSelection={ handleIntegrateSelection }
                                    />
                                </Grid.Column>
                            </Grid.Row>
                            <Grid.Row>
                                <Grid.Column width={ 13 }>
                                    <Divider hidden className="mt-0"/>
                                    { resolveQuickStartMode() }
                                </Grid.Column>
                            </Grid.Row>
                        </>
                    )
            }
        </Grid>
    );
};

/**
 * Default props for the component
 */
SinglePageApplicationQuickStart.defaultProps = {
    "data-testid": "spa-quick-start"
};

/**
 * A default export was added to support React.lazy.
 * TODO: Change this to a named export once react starts supporting named exports for code splitting.
 * @see {@link https://reactjs.org/docs/code-splitting.html#reactlazy}
 */
export default SinglePageApplicationQuickStart;
